@@ -219,14 +219,9 @@ katze_property_proxy (gpointer     object,
             string = g_strdup (G_PARAM_SPEC_STRING (pspec)->default_value);
         gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (widget),
                                        string ? string : "");
-        #if GTK_CHECK_VERSION (2, 12, 0)
-        g_signal_connect (widget, "file-set",
-                          G_CALLBACK (proxy_file_file_set_cb), object);
-        #else
         if (pspec->flags & G_PARAM_WRITABLE)
             g_signal_connect (widget, "selection-changed",
                               G_CALLBACK (proxy_file_file_set_cb), object);
-        #endif
     }
     else if (type == G_TYPE_PARAM_STRING && _hint == g_intern_string ("folder"))
     {
@@ -238,14 +233,9 @@ katze_property_proxy (gpointer     object,
             string = g_strdup (G_PARAM_SPEC_STRING (pspec)->default_value);
         gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (widget),
                                              string ? string : "");
-        #if GTK_CHECK_VERSION (2, 12, 0)
-        g_signal_connect (widget, "file-set",
-                          G_CALLBACK (proxy_folder_file_set_cb), object);
-        #else
         if (pspec->flags & G_PARAM_WRITABLE)
             g_signal_connect (widget, "selection-changed",
                               G_CALLBACK (proxy_folder_file_set_cb), object);
-        #endif
     }
     else if (type == G_TYPE_PARAM_STRING && _hint == g_intern_string ("uri"))
     {
