@@ -28,12 +28,9 @@ gtk_icon_entry_set_icon_from_pixbuf (GtkEntry*            entry,
                                      GdkPixbuf*           pixbuf)
 {
     /* Without this ugly hack pixbuf icons don't work */
-    if (pixbuf)
-    {
-        gtk_widget_hide (GTK_WIDGET (entry));
-        gtk_entry_set_icon_from_pixbuf (entry, position, pixbuf);
-        gtk_widget_show (GTK_WIDGET (entry));
-    }
+    gtk_widget_hide (GTK_WIDGET (entry));
+    gtk_entry_set_icon_from_pixbuf (entry, position, pixbuf);
+    gtk_widget_show (GTK_WIDGET (entry));
 }
 
 #else
@@ -1266,6 +1263,10 @@ gtk_icon_entry_set_icon_from_stock (GtkIconEntry *entry,
 {
   GdkPixbuf *pixbuf;
 
+  /* FIXME: Due to a bug in GtkIconEntry we need to set a non-NULL icon */
+  if (! stock_id)
+    stock_id = GTK_STOCK_INFO;
+
   pixbuf = gtk_widget_render_icon (GTK_WIDGET (entry),
 				   stock_id,
 				   GTK_ICON_SIZE_MENU,
