@@ -117,10 +117,9 @@ midori_history_get_stock_id (MidoriViewable* viewable)
 static void
 midori_history_add_clicked_cb (GtkWidget* toolitem)
 {
-    GtkWidget* browser = gtk_widget_get_toplevel (toolitem);
+    MidoriBrowser* browser = midori_browser_get_for_widget (toolitem);
     /* FIXME: Take selected folder into account */
-    midori_browser_edit_bookmark_dialog_new (MIDORI_BROWSER (browser),
-                                             NULL, TRUE, FALSE);
+    midori_browser_edit_bookmark_dialog_new (browser, NULL, TRUE, FALSE);
 }
 
 static void
@@ -152,11 +151,11 @@ static void
 midori_history_clear_clicked_cb (GtkWidget*     toolitem,
                                  MidoriHistory* history)
 {
-    GtkWidget* browser;
+    MidoriBrowser* browser;
     GtkWidget* dialog;
     gint result;
 
-    browser = gtk_widget_get_toplevel (GTK_WIDGET (history));
+    browser = midori_browser_get_for_widget (GTK_WIDGET (history));
     dialog = gtk_message_dialog_new (GTK_WINDOW (browser),
         GTK_DIALOG_DESTROY_WITH_PARENT,
         GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO,
@@ -275,7 +274,8 @@ midori_history_clear_cb (KatzeArray*    array,
 
 static void
 midori_history_disconnect_folder (MidoriHistory* history,
-                                  KatzeArray*    array)
+                                  KatzeArray*    array,
+                                  gboolean       unref)
 {
     KatzeItem* item;
     guint i;
@@ -293,8 +293,9 @@ midori_history_disconnect_folder (MidoriHistory* history,
     while ((item = katze_array_get_nth_item (array, i++)))
     {
         if (KATZE_IS_ARRAY (item))
-            midori_history_disconnect_folder (history, KATZE_ARRAY (item));
-        g_object_unref (item);
+            midori_history_disconnect_folder (history, KATZE_ARRAY (item), unref);
+        if (unref)
+            g_object_unref (item);
     }
 }
 
@@ -309,6 +310,7 @@ midori_history_add_item_cb (KatzeArray*    array,
 
     g_return_if_fail (KATZE_IS_ARRAY (array));
     g_return_if_fail (KATZE_IS_ITEM (added_item));
+    g_return_if_fail (MIDORI_IS_HISTORY (history));
 
     if (KATZE_IS_ARRAY (added_item))
     {
@@ -392,7 +394,7 @@ midori_history_remove_item_cb (KatzeArray*    array,
     g_assert (KATZE_IS_ITEM (removed_item));
 
     if (KATZE_IS_ARRAY (removed_item))
-        midori_history_disconnect_folder (history, KATZE_ARRAY (removed_item));
+        midori_history_disconnect_folder (history, KATZE_ARRAY (removed_item), TRUE);
 
     model = gtk_tree_view_get_model (GTK_TREE_VIEW (history->treeview));
     midori_history_remove_iter (model, NULL, removed_item);
@@ -424,7 +426,7 @@ midori_history_clear_cb (KatzeArray*    array,
             midori_history_remove_item_cb (array, item, history);
     }
 
-    midori_history_disconnect_folder (history, array);
+    midori_history_disconnect_folder (history, array, TRUE);
 }
 
 static void
@@ -459,7 +461,6 @@ midori_history_insert_item (MidoriHistory* history,
             age = day - pday;
             gtk_tree_store_insert_with_values (treestore, &iter, parent,
                                                0, 0, item, 1, age, -1);
-            g_object_unref (item);
             piter = &iter;
         }
         i = 0;
@@ -481,7 +482,7 @@ midori_history_set_app (MidoriHistory* history,
 
     if (history->array)
     {
-        midori_history_disconnect_folder (history, history->array);
+        midori_history_disconnect_folder (history, history->array, TRUE);
         g_object_unref (history->array);
         model = gtk_tree_view_get_model (GTK_TREE_VIEW (history->treeview));
         gtk_tree_store_clear (GTK_TREE_STORE (model));
@@ -499,7 +500,7 @@ midori_history_set_app (MidoriHistory* history,
 
         model = gtk_tree_view_get_model (GTK_TREE_VIEW (history->treeview));
         midori_history_insert_item (history, GTK_TREE_STORE (model),
-            NULL, KATZE_ITEM (g_object_ref (history->array)), day);
+            NULL, KATZE_ITEM (history->array), day);
     }
 }
 
@@ -597,13 +598,13 @@ midori_history_treeview_render_text_cb (GtkTreeViewColumn* column,
         }
         else if (age > 6)
         {
-            sdate = g_strdup_printf (_("A week ago"));
+            sdate = _("A week ago");
             g_object_set (renderer, "text", sdate, NULL);
-            g_free (sdate);
         }
         else if (age > 1)
         {
-            sdate = g_strdup_printf (_("%d days ago"), (gint)age);
+            sdate = g_strdup_printf (ngettext ("%d day ago",
+                "%d days ago", (gint)age), (gint)age);
             g_object_set (renderer, "text", sdate, NULL);
             g_free (sdate);
         }
@@ -641,10 +642,10 @@ midori_history_row_activated_cb (GtkTreeView*       treeview,
         uri = katze_item_get_uri (item);
         if (uri && *uri)
         {
-            GtkWidget* browser;
+            MidoriBrowser* browser;
 
-            browser = gtk_widget_get_toplevel (GTK_WIDGET (history));
-            midori_browser_set_current_uri (MIDORI_BROWSER (browser), uri);
+            browser = midori_browser_get_for_widget (GTK_WIDGET (history));
+            midori_browser_set_current_uri (browser, uri);
         }
 
         g_object_unref (item);
@@ -691,8 +692,8 @@ midori_history_open_activate_cb (GtkWidget*     menuitem,
 
     if (uri && *uri)
     {
-        GtkWidget* browser = gtk_widget_get_toplevel (GTK_WIDGET (history));
-        midori_browser_set_current_uri (MIDORI_BROWSER (browser), uri);
+        MidoriBrowser* browser = midori_browser_get_for_widget (GTK_WIDGET (history));
+        midori_browser_set_current_uri (browser, uri);
     }
 }
 
@@ -714,14 +715,14 @@ midori_history_open_in_tab_activate_cb (GtkWidget*     menuitem,
         {
             if ((uri = katze_item_get_uri (child)) && *uri)
             {
-                GtkWidget* browser;
+                MidoriBrowser* browser;
                 MidoriWebSettings* settings;
 
-                browser = gtk_widget_get_toplevel (GTK_WIDGET (history));
-                n = midori_browser_add_item (MIDORI_BROWSER (browser), child);
+                browser = midori_browser_get_for_widget (GTK_WIDGET (history));
+                n = midori_browser_add_item (browser, child);
                 settings = katze_object_get_object (browser, "settings");
                 if (!katze_object_get_boolean (settings, "open-tabs-in-the-background"))
-                    midori_browser_set_current_page (MIDORI_BROWSER (browser), n);
+                    midori_browser_set_current_page (browser, n);
                 g_object_unref (settings);
             }
             i++;
@@ -731,14 +732,14 @@ midori_history_open_in_tab_activate_cb (GtkWidget*     menuitem,
     {
         if ((uri = katze_item_get_uri (item)) && *uri)
         {
-            GtkWidget* browser;
+            MidoriBrowser* browser;
             MidoriWebSettings* settings;
 
-            browser = gtk_widget_get_toplevel (GTK_WIDGET (history));
-            n = midori_browser_add_item (MIDORI_BROWSER (browser), item);
+            browser = midori_browser_get_for_widget (GTK_WIDGET (history));
+            n = midori_browser_add_item (browser, item);
             settings = katze_object_get_object (browser, "settings");
             if (!katze_object_get_boolean (settings, "open-tabs-in-the-background"))
-                midori_browser_set_current_page (MIDORI_BROWSER (browser), n);
+                midori_browser_set_current_page (browser, n);
             g_object_unref (settings);
         }
     }
@@ -756,7 +757,7 @@ midori_history_open_in_window_activate_cb (GtkWidget*     menuitem,
 
     if (uri && *uri)
     {
-        GtkWidget* browser = gtk_widget_get_toplevel (GTK_WIDGET (history));
+        MidoriBrowser* browser = midori_browser_get_for_widget (GTK_WIDGET (history));
         g_signal_emit_by_name (browser, "new-window", uri);
     }
 }
@@ -773,8 +774,8 @@ midori_history_bookmark_activate_cb (GtkWidget*     menuitem,
 
     if (uri && *uri)
     {
-        GtkWidget* browser = gtk_widget_get_toplevel (GTK_WIDGET (history));
-        midori_browser_edit_bookmark_dialog_new (MIDORI_BROWSER (browser), item, TRUE, FALSE);
+        MidoriBrowser* browser = midori_browser_get_for_widget (GTK_WIDGET (history));
+        midori_browser_edit_bookmark_dialog_new (browser, item, TRUE, FALSE);
     }
 }
 
@@ -852,12 +853,12 @@ midori_history_button_release_event_cb (GtkWidget*      widget,
 
             if (uri && *uri)
             {
-                GtkWidget* browser;
+                MidoriBrowser* browser;
                 gint n;
 
-                browser = gtk_widget_get_toplevel (widget);
-                n = midori_browser_add_uri (MIDORI_BROWSER (browser), uri);
-                midori_browser_set_current_page (MIDORI_BROWSER (browser), n);
+                browser = midori_browser_get_for_widget (widget);
+                n = midori_browser_add_uri (browser, uri);
+                midori_browser_set_current_page (browser, n);
             }
         }
         else
@@ -967,6 +968,10 @@ midori_history_finalize (GObject* object)
 
     if (history->app)
         g_object_unref (history->app);
+
+    /* FIXME: We don't unref items (last argument is FALSE) because
+       our reference counting is incorrect. */
+    midori_history_disconnect_folder (history, history->array, FALSE);
     g_object_unref (history->array);
 }
 
